import random

import numpy
import tensorflow
import torch
import pandas
import pytest

from . import utils
from wandb.util import json_friendly


def pt_variable(nested_list, requires_grad=True):
	v = torch.autograd.Variable(utils.pytorch_tensor(nested_list))
	v.requires_grad = requires_grad
	return v


def r():
	return random.random()


def l(*shape):
	"""Makes a nested list of lists with a "shape" argument like numpy,
	TensorFlow, etc.
	"""
	if not shape:
		# reduce precision so we can use == for comparison regardless
		# of conversions between other libraries
		return float(numpy.float16(random.random()))
	else:
		return [l(*shape[1:]) for _ in range(shape[0])]


def json_friendly_test(orig_data, obj):
	data, converted = json_friendly(obj)
	utils.assert_deep_lists_equal(orig_data, data)
	assert converted


def tensorflow_json_friendly_test(orig_data):
	with tensorflow.Session().as_default() as s:
		json_friendly_test(orig_data, tensorflow.convert_to_tensor(orig_data))
		v = tensorflow.Variable(tensorflow.convert_to_tensor(orig_data))
		s.run(tensorflow.global_variables_initializer())
		json_friendly_test(orig_data, v)


@pytest.mark.skipif(utils.PYTORCH_VERSION < (0, 4), reason='0d tensors not supported until 0.4')
def test_pytorch_json_0d():
	a = l()
	json_friendly_test(a, utils.pytorch_tensor(a))
	json_friendly_test(a, pt_variable(a))


def test_pytorch_json_1d_1x1():
	a = l(1)
	json_friendly_test(a, utils.pytorch_tensor(a))
	json_friendly_test(a, pt_variable(a))


def test_pytorch_json_1d():
	a = l(3)
	json_friendly_test(a, utils.pytorch_tensor(a))
	json_friendly_test(a, pt_variable(a))


def test_pytorch_json_1d_large():
	a = l(300)
	json_friendly_test(a, utils.pytorch_tensor(a))
	json_friendly_test(a, pt_variable(a))


def test_pytorch_json_2d():
	a = l(3, 3)
	json_friendly_test(a, utils.pytorch_tensor(a))
	json_friendly_test(a, pt_variable(a))


def test_pytorch_json_2d_large():
	a = l(300, 300)
	json_friendly_test(a, utils.pytorch_tensor(a))
	json_friendly_test(a, pt_variable(a))


def test_pytorch_json_3d():
	a = l(3, 3, 3)
	json_friendly_test(a, utils.pytorch_tensor(a))
	json_friendly_test(a, pt_variable(a))


def test_pytorch_json_4d():
	a = l(3, 3, 3, 3)
	json_friendly_test(a, utils.pytorch_tensor(a))
	json_friendly_test(a, pt_variable(a))


def test_pytorch_json_nd():
	a = l(1, 1, 1, 1, 1, 1, 1, 1)
	json_friendly_test(a, utils.pytorch_tensor(a))
	json_friendly_test(a, pt_variable(a))


def test_pytorch_json_nd_large():
	a = l(3, 3, 3, 3, 3, 3, 3, 3)
	json_friendly_test(a, utils.pytorch_tensor(a))
	json_friendly_test(a, pt_variable(a))


def test_tensorflow_json_0d():
	tensorflow_json_friendly_test(l())


def test_tensorflow_json_1d_1x1():
	tensorflow_json_friendly_test(l(1))


def test_tensorflow_json_1d():
	tensorflow_json_friendly_test(l(3))


def test_tensorflow_json_1d_large():
	tensorflow_json_friendly_test(l(300))


def test_tensorflow_json_2d():
	tensorflow_json_friendly_test(l(3, 3))


def test_tensorflow_json_2d_large():
	tensorflow_json_friendly_test(l(300, 300))


def test_tensorflow_json_nd():
	tensorflow_json_friendly_test(l(1, 1, 1, 1, 1, 1, 1, 1))


def test_tensorflow_json_nd_large():
	tensorflow_json_friendly_test(l(3, 3, 3, 3, 3, 3, 3, 3))


def test_pandas_json_2d():
	a = l(3, 3)
	json_friendly_test(a, pandas.DataFrame(a))


def test_pandas_json_2d_large():
	a = l(300, 300)
	json_friendly_test(a, pandas.DataFrame(a))



import wandb
import numpy as np
import pytest
import PIL
import os
import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt
from click.testing import CliRunner

data = np.random.randint(255, size=(1000))


def test_raw_data():
    wbhist = wandb.Histogram(data)
    assert len(wbhist.histogram) == 64


def test_np_histogram():
    wbhist = wandb.Histogram(np_histogram=np.histogram(data))
    assert len(wbhist.histogram) == 10


def test_manual_histogram():
    wbhist = wandb.Histogram(np_histogram=([1, 2, 4], [3, 10, 20, 0]))
    assert len(wbhist.histogram) == 3


def test_fucked_up_histogram():
    with pytest.raises(ValueError):
        wbhist = wandb.Histogram(np_histogram=([1, 2, 3], [1]))


def test_transform():
    wbhist = wandb.Histogram(data)
    json = wandb.Histogram.transform(wbhist)
    assert json["_type"] == "histogram"
    assert len(json["values"]) == 64


image = np.random.randint(255, size=(28, 28))


def test_captions():
    wbone = wandb.Image(image, caption="Cool")
    wbtwo = wandb.Image(image, caption="Nice")
    assert wandb.Image.captions([wbone, wbtwo]) == ["Cool", "Nice"]


def test_transform():
    with CliRunner().isolated_filesystem():
        meta = wandb.Image.transform([wandb.Image(image)], ".", "test.jpg")
        assert meta == {'_type': 'images',
                        'count': 1, 'height': 28, 'width': 28}
        assert os.path.exists("media/images/test.jpg")


def test_guess_mode():
    image = np.random.randint(255, size=(28, 28, 3))
    wbimg = wandb.Image(image)
    assert wbimg.image.mode == "RGB"


def test_pil():
    pil = PIL.Image.new("L", (28, 28))
    img = wandb.Image(pil)
    assert img.image == pil


def test_matplotlib_image():
    plt.plot([1, 2, 2, 4])
    img = wandb.Image(plt)
    assert img.image.width == 640


def test_table_default():
    table = wandb.Table()
    table.add_row("Some awesome text", "Positive", "Negative")
    assert wandb.Table.transform(table) == {"_type": "table",
                                            "data": [["Some awesome text", "Positive", "Negative"]],
                                            "columns": ["Input", "Output", "Expected"]}


def test_table_custom():
    table = wandb.Table(["Foo", "Bar"])
    table.add_row("So", "Cool")
    table.add_row("&", "Rad")
    assert wandb.Table.transform(table) == {"_type": "table",
                                            "data": [["So", "Cool"], ["&", "Rad"]],
                                            "columns": ["Foo", "Bar"]}
